@@ -33,38 +33,27 @@
 #include <sys/resource.h>
 #endif
 
-#ifdef HAVE_OPENSSL
-#include <openssl/opensslv.h>
-#endif
-
-#include <vorbis/codec.h>
-
-#ifdef HAVE_THEORA
-#include <theora/theora.h>
-#endif
-
-#ifdef HAVE_SPEEX
-#include <speex/speex.h>
-#endif
-
-#ifdef HAVE_CURL
-#include <curl/curlver.h>
-#include <curl/curl.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
 #endif
 
 #ifdef HAVE_UNAME
 #include <sys/utsname.h>
 #endif
 
+#include <igloo/error.h>
+
 #include "common/net/sock.h"
 
 #include "admin.h"
+#include "version.h"
 #include "compat.h"
 #include "cfgfile.h"
 #include "connection.h"
 #include "listensocket.h"
 #include "refbuf.h"
 #include "client.h"
+#include "event_stream.h"
 #include "source.h"
 #include "global.h"
 #include "stats.h"
@@ -74,6 +63,7 @@
 #include "reportxml.h"
 #include "reportxml_helper.h"
 #include "xml2json.h"
+#include "util_crypt.h"
 
 #include "format.h"
 
@@ -137,6 +127,8 @@
 #define KILLSOURCE_RAW_REQUEST              "killsource"
 #define KILLSOURCE_HTML_REQUEST             "killsource.xsl"
 #define KILLSOURCE_JSON_REQUEST             "killsource.json"
+#define DUMPFILECONTROL_RAW_REQUEST         "dumpfilecontrol"
+#define DUMPFILECONTROL_HTML_REQUEST        "dumpfilecontrol.xsl"
 #define ADMIN_XSL_RESPONSE                  "response.xsl"
 #define MANAGEAUTH_RAW_REQUEST              "manageauth"
 #define MANAGEAUTH_HTML_REQUEST             "manageauth.xsl"
@@ -158,12 +150,13 @@
 #define DEFAULT_RAW_REQUEST                 ""
 #define DEFAULT_HTML_REQUEST                ""
 #define BUILDM3U_RAW_REQUEST                "buildm3u"
+#define EVENTSTREAM_RAW_REQUEST             "eventfeed"
 
-typedef enum {
-    ADMIN_DASHBOARD_STATUS_OK = 0,
-    ADMIN_DASHBOARD_STATUS_WARNING = 1,
-    ADMIN_DASHBOARD_STATUS_ERROR = 2
-} admin_dashboard_status_t;
+typedef struct {
+    size_t listeners;
+    size_t tls;
+    size_t ipv6;
+} country_t;
 
 typedef struct {
     const char *prefix;
@@ -184,9 +177,11 @@ static void command_list_listen_sockets (client_t *client, source_t *source, adm
 static void command_move_clients        (client_t *client, source_t *source, admin_format_t response);
 static void command_kill_client         (client_t *client, source_t *source, admin_format_t response);
 static void command_kill_source         (client_t *client, source_t *source, admin_format_t response);
+static void command_dumpfile_control    (client_t *client, source_t *source, admin_format_t response);
 static void command_manageauth          (client_t *client, source_t *source, admin_format_t response);
 static void command_updatemetadata      (client_t *client, source_t *source, admin_format_t response);
 static void command_buildm3u            (client_t *client, source_t *source, admin_format_t response);
+static void command_eventstream         (client_t *client, source_t *source, admin_format_t response);
 static void command_show_log            (client_t *client, source_t *source, admin_format_t response);
 static void command_mark_log            (client_t *client, source_t *source, admin_format_t response);
 static void command_dashboard           (client_t *client, source_t *source, admin_format_t response);
@@ -231,6 +226,8 @@ static const admin_command_handler_t handlers[] = {
     { KILLSOURCE_RAW_REQUEST,               ADMINTYPE_MOUNT,        ADMIN_FORMAT_RAW,           ADMINSAFE_UNSAFE,   command_kill_source, NULL},
     { KILLSOURCE_HTML_REQUEST,              ADMINTYPE_MOUNT,        ADMIN_FORMAT_HTML,          ADMINSAFE_UNSAFE,   command_kill_source, NULL},
     { KILLSOURCE_JSON_REQUEST,              ADMINTYPE_MOUNT,        ADMIN_FORMAT_JSON,          ADMINSAFE_UNSAFE,   command_kill_source, NULL},
+    { DUMPFILECONTROL_RAW_REQUEST,          ADMINTYPE_MOUNT,        ADMIN_FORMAT_RAW,           ADMINSAFE_UNSAFE,   command_dumpfile_control, NULL},
+    { DUMPFILECONTROL_HTML_REQUEST,         ADMINTYPE_MOUNT,        ADMIN_FORMAT_HTML,          ADMINSAFE_UNSAFE,   command_dumpfile_control, NULL},
     { MANAGEAUTH_RAW_REQUEST,               ADMINTYPE_GENERAL,      ADMIN_FORMAT_RAW,           ADMINSAFE_HYBRID,   command_manageauth, NULL},
     { MANAGEAUTH_HTML_REQUEST,              ADMINTYPE_GENERAL,      ADMIN_FORMAT_HTML,          ADMINSAFE_HYBRID,   command_manageauth, NULL},
     { MANAGEAUTH_JSON_REQUEST,              ADMINTYPE_GENERAL,      ADMIN_FORMAT_JSON,          ADMINSAFE_HYBRID,   command_manageauth, NULL},
@@ -238,6 +235,7 @@ static const admin_command_handler_t handlers[] = {
     { UPDATEMETADATA_HTML_REQUEST,          ADMINTYPE_MOUNT,        ADMIN_FORMAT_HTML,          ADMINSAFE_SAFE,     command_updatemetadata, NULL},
     { UPDATEMETADATA_JSON_REQUEST,          ADMINTYPE_MOUNT,        ADMIN_FORMAT_JSON,          ADMINSAFE_SAFE,     command_updatemetadata, NULL},
     { BUILDM3U_RAW_REQUEST,                 ADMINTYPE_MOUNT,        ADMIN_FORMAT_RAW,           ADMINSAFE_SAFE,     command_buildm3u, NULL},
+    { EVENTSTREAM_RAW_REQUEST,              ADMINTYPE_HYBRID,       ADMIN_FORMAT_RAW,           ADMINSAFE_SAFE,     command_eventstream, NULL},
     { SHOWLOG_RAW_REQUEST,                  ADMINTYPE_GENERAL,      ADMIN_FORMAT_RAW,           ADMINSAFE_SAFE,     command_show_log, NULL},
     { SHOWLOG_HTML_REQUEST,                 ADMINTYPE_GENERAL,      ADMIN_FORMAT_HTML,          ADMINSAFE_SAFE,     command_show_log, NULL},
     { SHOWLOG_JSON_REQUEST,                 ADMINTYPE_GENERAL,      ADMIN_FORMAT_JSON,          ADMINSAFE_SAFE,     command_show_log, NULL},
@@ -465,6 +463,33 @@ xmlNodePtr admin_build_rootnode(xmlDocPtr doc, const char *name)
     return rootnode;
 }
 
+static inline void admin_build_sourcelist__add_flag(xmlNodePtr parent, source_flags_t flags, source_flags_t flag, bool invert, const char *name, const char *description)
+{
+    xmlNodePtr node;
+    source_flags_t testflags = SOURCE_FLAGS_GOOD|flag;
+
+    if (invert ? (flags & flag) : !(flags & flag))
+        return;
+
+    node = xmlNewTextChild(parent, NULL, XMLSTR("flag"), XMLSTR(description));
+    xmlSetProp(node, XMLSTR("value"), XMLSTR(name));
+
+    if (invert)
+        testflags &= ~flag;
+
+    switch (source_get_health_by_flags(testflags)) {
+        case HEALTH_OK:
+            xmlSetProp(node, XMLSTR("maintenance-level"), XMLSTR("info"));
+            break;
+        case HEALTH_WARNING:
+            xmlSetProp(node, XMLSTR("maintenance-level"), XMLSTR("warning"));
+            break;
+        case HEALTH_ERROR:
+            xmlSetProp(node, XMLSTR("maintenance-level"), XMLSTR("error"));
+            break;
+    }
+}
+
 /* build an XML doc containing information about currently running sources.
  * If a mountpoint is passed then that source will not be added to the XML
  * doc even if the source is running */
@@ -525,6 +550,9 @@ xmlDocPtr admin_build_sourcelist(const char *mount, client_t *client, admin_form
             config_release_config();
 
             if (source->running) {
+                const source_flags_t flags = source->flags;
+                xmlNodePtr maintenancenode;
+
                 if (source->client) {
                     snprintf(buf, sizeof(buf), "%lu",
                         (unsigned long)(now - source->con->con_time));
@@ -536,7 +564,34 @@ xmlDocPtr admin_build_sourcelist(const char *mount, client_t *client, admin_form
                 }
                 xmlNewTextChild(srcnode, NULL, XMLSTR("content-type"),
                     XMLSTR(source->format->contenttype));
+
+                switch (source_get_health(source)) {
+                    case HEALTH_OK:
+                        xmlNewTextChild(srcnode, NULL, XMLSTR("health"), XMLSTR("green"));
+                        break;
+                    case HEALTH_WARNING:
+                        xmlNewTextChild(srcnode, NULL, XMLSTR("health"), XMLSTR("yellow"));
+                        break;
+                    case HEALTH_ERROR:
+                        xmlNewTextChild(srcnode, NULL, XMLSTR("health"), XMLSTR("red"));
+                        break;
+                }
+                maintenancenode = xmlNewChild(srcnode, NULL, XMLSTR("maintenance"), NULL);
+                xmlSetProp(maintenancenode, XMLSTR("comment"), XMLSTR("This is an experimental node. Do not use!"));
+
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_GOT_DATA, true, "no-got-data", "No data has yet been received.");
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_FORMAT_GENERIC, false, "format-generic", "Legacy or unsupported streaming format is used.");
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_LEGACY_METADATA, false, "legacy-metadata", "Legacy metadata on non-legacy source received. This is likely a bug in the source client.");
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_AGED, true, "no-aged", "The stream did not mature yet.");
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_HTTP_1_0_LISTENER, false, "http-1.0-listener", "Legacy HTTP/1.0 listener");
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_NOHOST_LISTENER, false, "nohost-listener", "Listener without useable Host:-header");
+                admin_build_sourcelist__add_flag(maintenancenode, flags, SOURCE_FLAG_NO_LANGUAGE, false, "nolanguage", "No content language set");
             }
+
+            snprintf(buf, sizeof(buf), "%"PRIu64, source->dumpfile_written);
+            xmlNewTextChild(srcnode, NULL, XMLSTR("dumpfile_written"), XMLSTR(buf));
+
+            admin_add_geoip_to_mount(source, srcnode, client->mode);
         }
         node = avl_get_next(node);
     }
@@ -882,6 +937,39 @@ static inline xmlNodePtr __add_listener(client_t        *client,
 
     xmlNewTextChild(node, NULL, XMLSTR("protocol"), XMLSTR(client_protocol_to_string(client->protocol)));
 
+    if (client->con) {
+        connection_t *con = client->con;
+        const char *sock;
+
+        if (*con->geoip.iso_3166_1_alpha_2 || con->geoip.have_latitude || con->geoip.have_longitude) {
+            xmlNodePtr geoip = xmlNewChild(node, NULL, XMLSTR("geoip"), NULL);
+
+            if (*con->geoip.iso_3166_1_alpha_2) {
+                xmlNodePtr country = xmlNewChild(geoip, NULL, XMLSTR("country"), NULL);
+                xmlSetProp(country, XMLSTR("iso-alpha-2"), XMLSTR(con->geoip.iso_3166_1_alpha_2));
+            }
+            if (con->geoip.have_latitude || con->geoip.have_longitude) {
+                xmlNodePtr location = xmlNewChild(geoip, NULL, XMLSTR("location"), NULL);
+                if (con->geoip.have_latitude) {
+                    snprintf(buf, sizeof(buf), "%f", con->geoip.latitude);
+                    xmlSetProp(location, XMLSTR("latitude"), XMLSTR(buf));
+                }
+                if (con->geoip.have_longitude) {
+                    snprintf(buf, sizeof(buf), "%f", con->geoip.longitude);
+                    xmlSetProp(location, XMLSTR("longitude"), XMLSTR(buf));
+                }
+            }
+        }
+
+        sock = listensocket_get_listener(con->listensocket_real)->id;
+        if (sock)
+            xmlNewTextChild(node, NULL, XMLSTR("listensocketreal"), XMLSTR(sock));
+
+        sock = listensocket_get_listener(con->listensocket_effective)->id;
+        if (sock)
+            xmlNewTextChild(node, NULL, XMLSTR("listensocketeffective"), XMLSTR(sock));
+    }
+
     do {
         xmlNodePtr history = xmlNewChild(node, NULL, XMLSTR("history"), NULL);
         size_t i;
@@ -910,6 +998,80 @@ void admin_add_listeners_to_mount(source_t          *source,
     avl_tree_unlock(source->client_tree);
 }
 
+static void admin_add_geoip_to_mount__country(source_t          *source,
+                                       xmlNodePtr        parent,
+                                       operation_mode    mode,
+                                       const char        *code,
+                                       country_t         *country)
+{
+    if (country->listeners) {
+        xmlNodePtr node = xmlNewChild(parent, NULL, XMLSTR("country"), NULL);
+        char buf[22];
+
+        if (code)
+            xmlSetProp(node, XMLSTR("iso-alpha-2"), XMLSTR(code));
+
+        snprintf(buf, sizeof(buf), "%llu", (long long unsigned)country->listeners);
+        xmlNewTextChild(node, NULL, XMLSTR("listeners"), XMLSTR(buf));
+
+        snprintf(buf, sizeof(buf), "%llu", (long long unsigned)country->tls);
+        xmlNewTextChild(node, NULL, XMLSTR("tls"), XMLSTR(buf));
+
+        snprintf(buf, sizeof(buf), "%llu", (long long unsigned)country->ipv6);
+        xmlNewTextChild(node, NULL, XMLSTR("ipv6"), XMLSTR(buf));
+    }
+}
+
+void admin_add_geoip_to_mount(source_t          *source,
+                              xmlNodePtr        parent,
+                              operation_mode    mode)
+{
+    avl_node *client_node;
+    xmlNodePtr geoip = xmlNewChild(parent, NULL, XMLSTR("geoip"), NULL);
+    country_t countries[26][26];
+    country_t default_country;
+
+    memset(countries, 0, sizeof(countries));
+    memset(&default_country, 0, sizeof(default_country));
+
+    avl_tree_rlock(source->client_tree);
+    client_node = avl_get_first(source->client_tree);
+    while(client_node) {
+        client_t *client = client_node->key;
+        connection_t *con = client->con;
+        country_t *country = &default_country;
+
+        if (con && *con->geoip.iso_3166_1_alpha_2) {
+            const char *iso = client->con->geoip.iso_3166_1_alpha_2;
+
+            if ((iso[0] >= 'a' && iso[0] <= 'z') && (iso[1] >= 'a' && iso[1] <= 'z')) {
+                country = &(countries[iso[0] - 'a'][iso[1] - 'a']);
+            }
+        }
+        country->listeners++;
+
+        if (con) {
+            if (con->tls)
+                country->tls++;
+
+            if (con->ip && strchr(con->ip, ':'))
+                country->ipv6++;
+        }
+
+        client_node = avl_get_next(client_node);
+    }
+    avl_tree_unlock(source->client_tree);
+
+    for (size_t idx_a = 0; idx_a < 26; idx_a++) {
+        for (size_t idx_b = 0; idx_b < 26; idx_b++) {
+            const char code[3] = {idx_a + 'a', idx_b + 'a', 0};
+            admin_add_geoip_to_mount__country(source, geoip, mode, code, &(countries[idx_a][idx_b]));
+        }
+    }
+
+    admin_add_geoip_to_mount__country(source, geoip, mode, NULL, &default_country);
+}
+
 static void command_show_listeners(client_t *client,
                                    source_t *source,
                                    admin_format_t response)
@@ -930,6 +1092,7 @@ static void command_show_listeners(client_t *client,
     xmlNewTextChild(srcnode, NULL, XMLSTR(client->mode == OMODE_LEGACY ? "Listeners" : "listeners"), XMLSTR(buf));
 
     admin_add_listeners_to_mount(source, srcnode, client->mode);
+    admin_add_geoip_to_mount(source, srcnode, client->mode);
 
     admin_send_response(doc, client, response,
         LISTCLIENTS_HTML_REQUEST);
@@ -950,6 +1113,12 @@ static void command_buildm3u(client_t *client, source_t *source, admin_format_t
     client_send_buffer(client, 200, "audio/x-mpegurl", NULL, buffer, -1, "Content-Disposition: attachment; filename=listen.m3u\r\n");
 }
 
+static void command_eventstream         (client_t *client, source_t *source, admin_format_t response)
+{
+    (void)source, (void)response;
+    event_stream_add_client(client);
+}
+
 xmlNodePtr admin_add_role_to_authentication(auth_t *auth, xmlNodePtr parent)
 {
     xmlNodePtr rolenode = xmlNewChild(parent, NULL, XMLSTR("role"), NULL);
@@ -1110,6 +1279,19 @@ static void command_kill_source(client_t *client,
     admin_send_response_simple(client, source, response, "Source Removed", 1);
 }
 
+static void command_dumpfile_control    (client_t *client, source_t *source, admin_format_t response)
+{
+    const char *action;
+    COMMAND_REQUIRE(client, "action", action);
+
+    if (strcmp(action, "kill") == 0) {
+        source_kill_dumpfile(source);
+        admin_send_response_simple(client, source, response, "Dump file killed.", 1);
+    } else {
+        admin_send_response_simple(client, source, response, "No such action", 0);
+    }
+}
+
 static void command_kill_client(client_t *client,
                                 source_t *source,
                                 admin_format_t response)
@@ -1185,6 +1367,7 @@ static void command_metadata(client_t *client,
     if (source->parser && source->parser->req_type == httpp_req_put) {
         ICECAST_LOG_ERROR("Got legacy SOURCE-style metadata update command on "
             "source connected with PUT at mountpoint %H", source->mount);
+        source_set_flags(source, SOURCE_FLAG_LEGACY_METADATA);
     }
 
     COMMAND_REQUIRE(client, "mode", action);
@@ -1229,6 +1412,7 @@ static void command_metadata(client_t *client,
     } else {
         ICECAST_LOG_ERROR("Got legacy shoutcast-style metadata update command "
             "on source that does not accept it at mountpoint %H", source->mount);
+        source_set_flags(source, SOURCE_FLAG_LEGACY_METADATA);
 
         admin_send_response_simple(client, source, response, "Mountpoint will not accept URL updates", 1);
         return;
@@ -1287,12 +1471,22 @@ static void command_shoutcast_metadata(client_t *client,
 
 static void command_stats(client_t *client, source_t *source, admin_format_t response)
 {
-    unsigned int flags = (source) ? STATS_XML_FLAG_SHOW_HIDDEN|STATS_XML_FLAG_SHOW_LISTENERS : STATS_XML_FLAG_SHOW_HIDDEN;
+    unsigned int flags = STATS_XML_FLAG_SHOW_HIDDEN;
     const char *mount = (source) ? source->mount : NULL;
     xmlDocPtr doc;
+    const char *show_listeners;
 
     ICECAST_LOG_DEBUG("Stats request, sending xml stats");
 
+    COMMAND_OPTIONAL(client, "show-listeners", show_listeners);
+    if (show_listeners) {
+        if (util_str_to_bool(show_listeners))
+            flags |= STATS_XML_FLAG_SHOW_LISTENERS;
+    } else {
+        if (source)
+            flags |= STATS_XML_FLAG_SHOW_LISTENERS;
+    }
+
     doc = stats_get_xml(flags, mount, client);
     admin_send_response(doc, client, response, STATS_HTML_REQUEST);
     xmlFreeDoc(doc);
@@ -1380,6 +1574,11 @@ static void command_list_listen_sockets(client_t *client, source_t *source, admi
         reportxml_helper_add_value_enum(resource, "family", sock_family_to_string(listensocket_get_family(sockets[i])));
         reportxml_helper_add_value_string(resource, "id", listener->id);
         reportxml_helper_add_value_string(resource, "on_behalf_of", listener->on_behalf_of);
+        reportxml_helper_add_value_string(config, "client_address", listener->client_address);
+        reportxml_helper_add_value_string(config, "client_ip", listener->client_ip);
+
+        for (size_t i = 0; i < (sizeof(listener->trusted_proxy)/sizeof(listener->trusted_proxy[0])); i++)
+            reportxml_helper_add_value_string(config, "trusted_proxy", listener->trusted_proxy[i]);
 
         if (listener->port > 0) {
             reportxml_helper_add_value_int(config, "port", listener->port);
@@ -1602,19 +1801,10 @@ static void __reportxml_add_maintenance(reportxml_node_t *parent, reportxml_data
     refobject_unref(incident);
 }
 
-static admin_dashboard_status_t command_dashboard__atbest(admin_dashboard_status_t a, admin_dashboard_status_t b)
-{
-    if (a > b) {
-        return a;
-    } else {
-        return b;
-    }
-}
-
 #if HAVE_GETRLIMIT && HAVE_SYS_RESOURCE_H
-static admin_dashboard_status_t command_dashboard__getrlimit(ice_config_t *config, reportxml_node_t *parent, reportxml_database_t *db)
+static health_t command_dashboard__getrlimit(ice_config_t *config, reportxml_node_t *parent, reportxml_database_t *db)
 {
-    admin_dashboard_status_t ret = ADMIN_DASHBOARD_STATUS_OK;
+    health_t ret = HEALTH_OK;
     struct rlimit limit;
 
     if (getrlimit(RLIMIT_NOFILE, &limit) == 0) {
@@ -1624,7 +1814,7 @@ static admin_dashboard_status_t command_dashboard__getrlimit(ice_config_t *confi
                 // We assume that we need one FD per client, at max three per source (e.g. for auth), and at max 24 additional for logfiles and similar.
                 // This is just an estimation.
                 __reportxml_add_maintenance(parent, db, "a93a842a-9664-43a9-b707-7f358066fe2b", "error", "Global client, and source limit is bigger than suitable for current open file limit.", NULL);
-                ret = command_dashboard__atbest(ret, ADMIN_DASHBOARD_STATUS_ERROR);
+                ret = health_atbest(ret, HEALTH_ERROR);
             }
         }
     }
@@ -1647,7 +1837,7 @@ static void command_dashboard           (client_t *client, source_t *source, adm
     reportxml_t *report = client_get_reportxml("0aa76ea1-bf42-49d1-887e-ca95fb307dc4", NULL, NULL);
     reportxml_node_t *reportnode = reportxml_get_node_by_type(report, REPORTXML_NODE_TYPE_REPORT, 0);
     reportxml_node_t *incident = reportxml_get_node_by_type(report, REPORTXML_NODE_TYPE_INCIDENT, 0);
-    admin_dashboard_status_t status = ADMIN_DASHBOARD_STATUS_OK;
+    health_t health = HEALTH_OK;
     reportxml_node_t *resource;
     reportxml_node_t *node;
     bool has_sources;
@@ -1685,19 +1875,19 @@ static void command_dashboard           (client_t *client, source_t *source, adm
     reportxml_node_add_child(resource, node);
     refobject_unref(node);
 
-    if (config->config_problems || has_too_many_clients) {
-        status = command_dashboard__atbest(status, ADMIN_DASHBOARD_STATUS_ERROR);
-    } else if (!has_sources || has_many_clients || !inet6_enabled) {
-        status = command_dashboard__atbest(status, ADMIN_DASHBOARD_STATUS_WARNING);
+    if (config->config_problems || has_too_many_clients || !inet6_enabled) {
+        health = health_atbest(health, HEALTH_ERROR);
+    } else if (!has_sources || has_many_clients) {
+        health = health_atbest(health, HEALTH_WARNING);
     }
 
 #ifdef DEVEL_LOGGING
-    status = command_dashboard__atbest(status, ADMIN_DASHBOARD_STATUS_WARNING);
+    health = health_atbest(health, HEALTH_WARNING);
     __reportxml_add_maintenance(reportnode, config->reportxml_db, "c704804e-d3b9-4544-898b-d477078135de", "warning", "Developer logging is active. This mode is not for production.", NULL);
 #endif
 
     if (!inet6_enabled) {
-        __reportxml_add_maintenance(reportnode, config->reportxml_db, "f90219e1-bd07-4b54-b1ee-0ba6a0289a15", "warning", "IPv6 not enabled.", NULL);
+        __reportxml_add_maintenance(reportnode, config->reportxml_db, "f90219e1-bd07-4b54-b1ee-0ba6a0289a15", "error", "IPv6 not enabled.", NULL);
         if (sock_is_ipv4_mapped_supported()) {
             __reportxml_add_maintenance(reportnode, config->reportxml_db, "709ab43b-251d-49a5-a4fe-c749eaabf17c", "info", "IPv4-mapped IPv6 is available on this system.", NULL);
         }
@@ -1719,6 +1909,34 @@ static void command_dashboard           (client_t *client, source_t *source, adm
         __reportxml_add_maintenance(reportnode, config->reportxml_db, "0f6f757d-52d8-4b9a-8e57-9bcd528fffba", "error", "Invalid tags are used in the config file. See the error.log for details and update your configuration accordingly.", NULL);
     if (config->config_problems & CONFIG_PROBLEM_VALIDATION)
         __reportxml_add_maintenance(reportnode, config->reportxml_db, "8fc33086-274d-4ccb-b32f-599b3fa0f41a", "error", "The configuration did not validate. See the error.log for details and update your configuration accordingly.", NULL);
+    if (config->config_problems & CONFIG_PROBLEM_FILE_PERMISSION)
+        __reportxml_add_maintenance(reportnode, config->reportxml_db, "269e25fe-f16f-41a1-b826-4088b1b3fc23", "error", "File permissions are wrong. See the error.log for details and update your configuration accordingly. Future versions of Icecast may reject affected files.", NULL);
+    if (config->config_problems & CONFIG_PROBLEM_UNSAFE_FILE)
+        __reportxml_add_maintenance(reportnode, config->reportxml_db, "d810fa47-9d6a-4ff9-8d1b-854a75058866", "error", "Potentially unsafe file permissions. See the error.log for details and update your configuration accordingly. Future versions of Icecast may reject affected files.", NULL);
+
+    if (config->chroot) {
+#if HAVE_CHROOT
+        if (global.chroot_succeeded) {
+            __reportxml_add_maintenance(reportnode, config->reportxml_db, "6830cbf7-cd68-4c0c-ab5a-81499c70fd34", "info", "chroot configured and active.", NULL);
+        } else {
+            __reportxml_add_maintenance(reportnode, config->reportxml_db, "2d584a76-e67c-4268-b7e8-139b0b9b1131", "error", "chroot configured but failed.", NULL);
+        }
+#else
+        __reportxml_add_maintenance(reportnode, config->reportxml_db, "1a3fea5c-3352-4cb5-85cc-51ab9bd6ea83", "error", "chroot configured but not supported by operating system.", NULL);
+#endif
+    }
+
+    if(config->chuid) {
+#if HAVE_SETUID
+        if (global.chuid_succeeded) {
+            __reportxml_add_maintenance(reportnode, config->reportxml_db, "bab05e81-fd03-4773-9fc5-c4609883a5e3", "info", "Change of UID/GID configured and active.", NULL);
+        } else {
+            __reportxml_add_maintenance(reportnode, config->reportxml_db, "4f856dd4-7aac-44b4-95b5-b6798f547603", "error", "Change of UID/GID configured but failed.", NULL);
+        }
+#else
+        __reportxml_add_maintenance(reportnode, config->reportxml_db, "afcaa756-b91c-4496-a9e2-44400a18789c", "error", "Change of UID/GID configured but not supported by operating system.", NULL);
+#endif
+    }
 
     if (!has_sources)
         __reportxml_add_maintenance(reportnode, config->reportxml_db, "f68dd8a3-22b1-4118-aba6-b039f2c5b51e", "info", "Currently no sources are connected to this server.", NULL);
@@ -1733,21 +1951,31 @@ static void command_dashboard           (client_t *client, source_t *source, adm
     }
 
 #if HAVE_GETRLIMIT && HAVE_SYS_RESOURCE_H
-    status = command_dashboard__atbest(status, command_dashboard__getrlimit(config, reportnode, config->reportxml_db));
+    if (true) {
+        health_t limits = command_dashboard__getrlimit(config, reportnode, config->reportxml_db);
+        health = health_atbest(health, limits);
+    }
 #endif
 
-    switch (status) {
-        case ADMIN_DASHBOARD_STATUS_OK:
-            reportxml_helper_add_value_enum(resource, "status", "green");
-        break;
-        case ADMIN_DASHBOARD_STATUS_WARNING:
-            reportxml_helper_add_value_enum(resource, "status", "yellow");
-        break;
-        case ADMIN_DASHBOARD_STATUS_ERROR:
-            reportxml_helper_add_value_enum(resource, "status", "red");
-        break;
+    if (true) {
+        /* A list of environment variables that will normally not be seen in a daemon environment. */
+        static const char * const keys[] = {"DISPLAY", "LS_COLORS", "TERM", "XDG_RUNTIME_DIR"};
+        size_t i;
+
+        for (i = 0; i < (sizeof(keys)/sizeof(*keys)); i++) {
+            if (getenv(keys[i])) {
+                __reportxml_add_maintenance(reportnode, config->reportxml_db, "dc91ce96-f473-41d1-bfff-379666306911", "info", "Environment is noisy.", NULL);
+                break;
+            }
+        }
     }
 
+    if (!util_crypt_is_new_secure()) {
+        __reportxml_add_maintenance(reportnode, config->reportxml_db, "40d134e3-fbbe-46b1-a409-9b2ca8954528", "warning", "No secure password hash support detected.", NULL);
+    }
+
+    reportxml_helper_add_value_health(resource, "status", health);
+
     reportxml_node_add_child(incident, resource);
     refobject_unref(resource);
 
@@ -1759,16 +1987,6 @@ static void command_dashboard           (client_t *client, source_t *source, adm
     refobject_unref(report);
 }
 
-#ifdef HAVE_SPEEX
-static inline const char *get_speex_version(void)
-{
-    const char *version;
-    if (speex_lib_ctl(SPEEX_LIB_GET_VERSION_STRING, &version) != 0)
-        return NULL;
-    return version;
-}
-#endif
-
 static void command_version             (client_t *client, source_t *source, admin_format_t response)
 {
     reportxml_t      *report        = client_get_reportxml("8cdfc150-094d-42f7-9c61-f9fb9a6e07e7", NULL, NULL);
@@ -1780,62 +1998,24 @@ static void command_version             (client_t *client, source_t *source, adm
     reportxml_node_t *cflags        = reportxml_node_new(REPORTXML_NODE_TYPE_VALUE, NULL, NULL, NULL);
     reportxml_node_t *rflags        = reportxml_node_new(REPORTXML_NODE_TYPE_VALUE, NULL, NULL, NULL);
     ice_config_t *icecast_config;
-#ifdef HAVE_CURL
-    const curl_version_info_data * curl_runtime_version = curl_version_info(CURLVERSION_NOW);
-#endif
-    struct {
-        const char *name;
-        const char *compiletime;
-        const char *runtime;
-    } dependency_versions[] = {
-        {"libxml2", LIBXML_DOTTED_VERSION, NULL},
-#if defined(HAVE_OPENSSL) && defined(OPENSSL_VERSION_TEXT)
-        {"OpenSSL", OPENSSL_VERSION_TEXT, NULL},
-#endif
-        {"libvorbis", NULL, vorbis_version_string()},
-#ifdef HAVE_THEORA
-        {"libtheora", NULL, theora_version_string()},
-#endif
-#ifdef HAVE_SPEEX
-        {"libspeex", NULL, get_speex_version()},
-#endif
-#ifdef HAVE_CURL
-        {"libcurl", LIBCURL_VERSION, curl_runtime_version->version},
-#endif
-        {NULL, NULL, NULL}
-    };
-    const char *compiletime_flags[] = {
-#ifdef HAVE_POLL
-        "poll",
-#endif
-#ifdef HAVE_SYS_SELECT_H
-        "select",
-#endif
-#ifdef HAVE_UNAME
-        "uname",
-#endif
-#ifdef HAVE_GETHOSTNAME
-        "gethostname",
-#endif
-#ifdef HAVE_GETADDRINFO
-        "getaddrinfo",
-#endif
-#ifdef WIN32
-        "win32",
-#endif
-#ifdef DEVEL_LOGGING
-        "developer-logging",
-#endif
-        NULL,
-    };
+    const char * const * compiletime_flags = version_get_compiletime_flags();
+    const icecast_dependency_t * dependency_versions = version_get_dependencies();
     size_t i;
 
     reportxml_node_set_attribute(resource, "type", "result");
     reportxml_node_add_child(incident, resource);
 
     reportxml_helper_add_value_string(resource, "version", ICECAST_VERSION_STRING);
+#ifdef ICECAST_BUILD_INFO
+    reportxml_helper_add_value_string(resource, "build-info", ICECAST_BUILD_INFO);
+#endif
     reportxml_helper_add_value_int(resource, "address-bits", sizeof(void*)*8);
 
+#ifdef HAVE_SYS_SELECT_H
+    reportxml_helper_add_value_int(resource, "fd-set-size", FD_SETSIZE);
+    reportxml_helper_add_value_int(resource, "fd-set-object-size", sizeof(fd_set));
+#endif
+
 #ifdef HAVE_GETHOSTNAME
     if (true) {
         char hostname[80];
@@ -1891,6 +2071,8 @@ static void command_version             (client_t *client, source_t *source, adm
     reportxml_helper_add_value_flag(rflags, "cfgp-node-obsolete", icecast_config->config_problems & CONFIG_PROBLEM_OBSOLETE_NODE);
     reportxml_helper_add_value_flag(rflags, "cfgp-node-invalid", icecast_config->config_problems & CONFIG_PROBLEM_INVALID_NODE);
     reportxml_helper_add_value_flag(rflags, "cfgp-validation", icecast_config->config_problems & CONFIG_PROBLEM_VALIDATION);
+    reportxml_helper_add_value_flag(rflags, "cfgp-file-permission", icecast_config->config_problems & CONFIG_PROBLEM_FILE_PERMISSION);
+    reportxml_helper_add_value_flag(rflags, "cfgp-unsafe-file", icecast_config->config_problems & CONFIG_PROBLEM_UNSAFE_FILE);
     config_release_config();
 
     reportxml_node_set_attribute(dependencies, "type", "structure");
@@ -1929,6 +2111,16 @@ static void command_version             (client_t *client, source_t *source, adm
     reportxml_helper_add_value_flag(rflags, "bound-inet6", listensocket_container_is_family_included(global.listensockets, SOCK_FAMILY_INET6));
     global_unlock();
 
+    reportxml_helper_add_value_flag(rflags, "crypt-1", util_crypt_is_supported("$1$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-3", util_crypt_is_supported("$3$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-5", util_crypt_is_supported("$5$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-6", util_crypt_is_supported("$6$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-7", util_crypt_is_supported("$7$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-md5", util_crypt_is_supported("$md5$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-sha1", util_crypt_is_supported("$sha1$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-y", util_crypt_is_supported("$y$"));
+    reportxml_helper_add_value_flag(rflags, "crypt-gy", util_crypt_is_supported("$gy$"));
+
     refobject_unref(config);
     refobject_unref(dependencies);
     refobject_unref(cflags);
