@@ -3,14 +3,15 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org,
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2012-2018, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
-/** 
+/**
  * Client authentication functions
  */
 
@@ -29,8 +30,8 @@
 #include "source.h"
 #include "client.h"
 #include "cfgfile.h"
-#include "httpp/httpp.h"
-#include "md5.h"
+#include "common/httpp/httpp.h"
+#include "util_crypt.h"
 
 #include "logging.h"
 #define CATMODULE "auth_htpasswd"
@@ -44,8 +45,7 @@ static auth_result htpasswd_deleteuser(auth_t *auth, const char *username);
 static auth_result htpasswd_userlist(auth_t *auth, xmlNodePtr srcnode);
 static int _free_user (void *key);
 
-typedef struct
-{
+typedef struct {
     char *name;
     char *pass;
 } htpasswd_user;
@@ -57,7 +57,8 @@ typedef struct {
     time_t mtime;
 } htpasswd_auth_state;
 
-static void htpasswd_clear(auth_t *self) {
+static void htpasswd_clear(auth_t *self)
+{
     htpasswd_auth_state *state = self->state;
     free(state->filename);
     if (state->users)
@@ -67,32 +68,18 @@ static void htpasswd_clear(auth_t *self) {
 }
 
 
-/* md5 hash */
-static char *get_hash(const char *data, int len)
-{
-    struct MD5Context context;
-    unsigned char digest[16];
-
-    MD5Init(&context);
-
-    MD5Update(&context, (const unsigned char *)data, len);
-
-    MD5Final(digest, &context);
-
-    return util_bin_to_hex(digest, 16);
-}
-
-
-static int compare_users (void *arg, void *a, void *b)
+static int compare_users(void *arg, void *a, void *b)
 {
     htpasswd_user *user1 = (htpasswd_user *)a;
     htpasswd_user *user2 = (htpasswd_user *)b;
 
+    (void)arg;
+
     return strcmp (user1->name, user2->name);
 }
 
 
-static int _free_user (void *key)
+static int _free_user(void *key)
 {
     htpasswd_user *user = (htpasswd_user *)key;
 
@@ -102,7 +89,7 @@ static int _free_user (void *key)
 }
 
 
-static void htpasswd_recheckfile (htpasswd_auth_state *htpasswd)
+static void htpasswd_recheckfile(htpasswd_auth_state *htpasswd)
 {
     FILE *passwdfile;
     avl_tree *new_users;
@@ -113,8 +100,7 @@ static void htpasswd_recheckfile (htpasswd_auth_state *htpasswd)
 
     if (htpasswd->filename == NULL)
         return;
-    if (stat (htpasswd->filename, &file_stat) < 0)
-    {
+    if (stat (htpasswd->filename, &file_stat) < 0) {
         ICECAST_LOG_WARN("failed to check status of %s", htpasswd->filename);
 
         /* Create a dummy users tree for things to use later */
@@ -126,16 +112,14 @@ static void htpasswd_recheckfile (htpasswd_auth_state *htpasswd)
         return;
     }
 
-    if (file_stat.st_mtime == htpasswd->mtime)
-    {
+    if (file_stat.st_mtime == htpasswd->mtime) {
         /* common case, no update to file */
         return;
     }
     ICECAST_LOG_INFO("re-reading htpasswd file \"%s\"", htpasswd->filename);
     passwdfile = fopen (htpasswd->filename, "rb");
-    if (passwdfile == NULL)
-    {
-        ICECAST_LOG_WARN("Failed to open authentication database \"%s\": %s", 
+    if (passwdfile == NULL) {
+        ICECAST_LOG_WARN("Failed to open authentication database \"%s\": %s",
                 htpasswd->filename, strerror(errno));
         return;
     }
@@ -143,18 +127,16 @@ static void htpasswd_recheckfile (htpasswd_auth_state *htpasswd)
 
     new_users = avl_tree_new (compare_users, NULL);
 
-    while (get_line(passwdfile, line, MAX_LINE_LEN))
-    {
+    while (get_line(passwdfile, line, MAX_LINE_LEN)) {
         int len;
         htpasswd_user *entry;
 
         num++;
-        if(!line[0] || line[0] == '#')
+        if (!line[0] || line[0] == '#')
             continue;
 
         sep = strrchr (line, ':');
-        if (sep == NULL)
-        {
+        if (sep == NULL) {
             ICECAST_LOG_WARN("No separator on line %d (%s)", num, htpasswd->filename);
             continue;
         }
@@ -184,76 +166,69 @@ static auth_result htpasswd_auth (auth_client *auth_user)
     htpasswd_user entry;
     void *result;
 
-    if (client->username == NULL || client->password == NULL)
-        return AUTH_FAILED;
+    if (!client->username || !client->password)
+        return AUTH_NOMATCH;
 
-    if (htpasswd->filename == NULL)
-    {
+    if (!htpasswd->filename) {
         ICECAST_LOG_ERROR("No filename given in options for authenticator.");
-        return AUTH_FAILED;
+        return AUTH_NOMATCH;
     }
     htpasswd_recheckfile (htpasswd);
 
     if (htpasswd->users == NULL) {
         ICECAST_LOG_ERROR("No user list.");
-        return AUTH_FAILED;
+        return AUTH_NOMATCH;
     }
 
     thread_rwlock_rlock (&htpasswd->file_rwlock);
     entry.name = client->username;
-    if (avl_get_by_key (htpasswd->users, &entry, &result) == 0)
-    {
+    if (avl_get_by_key (htpasswd->users, &entry, &result) == 0) {
         htpasswd_user *found = result;
-        char *hashed_pw;
 
         thread_rwlock_unlock (&htpasswd->file_rwlock);
-        hashed_pw = get_hash (client->password, strlen (client->password));
-        if (strcmp (found->pass, hashed_pw) == 0)
-        {
-            free (hashed_pw);
+        if (util_crypt_check(client->password, found->pass)) {
             return AUTH_OK;
         }
-        free (hashed_pw);
-        ICECAST_LOG_DEBUG("incorrect password for client");
+        ICECAST_LOG_DEBUG("incorrect password for client with username: %s", client->username);
         return AUTH_FAILED;
     }
     ICECAST_LOG_DEBUG("no such username: %s", client->username);
     thread_rwlock_unlock (&htpasswd->file_rwlock);
-    return AUTH_FAILED;
+    return AUTH_NOMATCH;
 }
 
-
 int  auth_get_htpasswd_auth (auth_t *authenticator, config_options_t *options)
 {
     htpasswd_auth_state *state;
 
-    authenticator->authenticate = htpasswd_auth;
+    authenticator->authenticate_client = htpasswd_auth;
     authenticator->free = htpasswd_clear;
     authenticator->adduser = htpasswd_adduser;
     authenticator->deleteuser = htpasswd_deleteuser;
     authenticator->listuser = htpasswd_userlist;
+    authenticator->immediate = 1;
 
     state = calloc(1, sizeof(htpasswd_auth_state));
 
     while(options) {
-        if(!strcmp(options->name, "filename"))
-        {
+        if(!strcmp(options->name, "filename")) {
             free (state->filename);
             state->filename = strdup(options->value);
         }
         options = options->next;
     }
 
-    if (state->filename)
-        ICECAST_LOG_INFO("Configured htpasswd authentication using password file \"%s\"", 
+    if (state->filename) {
+        ICECAST_LOG_INFO("Configured htpasswd authentication using password file \"%s\"",
                 state->filename);
-    else
+    } else {
         ICECAST_LOG_ERROR("No filename given in options for authenticator.");
+    }
 
     authenticator->state = state;
 
     thread_rwlock_create(&state->file_rwlock);
-    htpasswd_recheckfile (state);
+    htpasswd_recheckfile(state);
 
     return 0;
 }
@@ -274,7 +249,7 @@ static auth_result htpasswd_adduser (auth_t *auth, const char *username, const c
 
     htpasswd_recheckfile (state);
 
-    if (state->filename == NULL) {
+    if (state->users == NULL) {
         ICECAST_LOG_ERROR("No user list.");
         return AUTH_FAILED;
     }
@@ -282,23 +257,21 @@ static auth_result htpasswd_adduser (auth_t *auth, const char *username, const c
     thread_rwlock_wlock (&state->file_rwlock);
 
     entry.name = (char*)username;
-    if (avl_get_by_key (state->users, &entry, &result) == 0)
-    {
+    if (avl_get_by_key (state->users, &entry, &result) == 0) {
         thread_rwlock_unlock (&state->file_rwlock);
         return AUTH_USEREXISTS;
     }
 
     passwdfile = fopen(state->filename, "ab");
 
-    if (passwdfile == NULL)
-    {
+    if (passwdfile == NULL) {
         thread_rwlock_unlock (&state->file_rwlock);
-        ICECAST_LOG_WARN("Failed to open authentication database \"%s\": %s", 
+        ICECAST_LOG_WARN("Failed to open authentication database \"%s\": %s",
                 state->filename, strerror(errno));
         return AUTH_FAILED;
     }
 
-    hashed_password = get_hash(password, strlen(password));
+    hashed_password = util_crypt_hash(password);
     if (hashed_password) {
         fprintf(passwdfile, "%s:%s\n", username, hashed_password);
         free(hashed_password);
@@ -324,7 +297,7 @@ static auth_result htpasswd_deleteuser(auth_t *auth, const char *username)
 
     state = auth->state;
 
-    if (state->filename == NULL) {
+    if (!state->filename) {
         ICECAST_LOG_ERROR("No filename given in options for authenticator.");
         return AUTH_FAILED;
     }
@@ -338,7 +311,7 @@ static auth_result htpasswd_deleteuser(auth_t *auth, const char *username)
     passwdfile = fopen(state->filename, "rb");
 
     if(passwdfile == NULL) {
-        ICECAST_LOG_WARN("Failed to open authentication database \"%s\": %s", 
+        ICECAST_LOG_WARN("Failed to open authentication database \"%s\": %s",
                 state->filename, strerror(errno));
         thread_rwlock_unlock (&state->file_rwlock);
         return AUTH_FAILED;
@@ -346,8 +319,7 @@ static auth_result htpasswd_deleteuser(auth_t *auth, const char *username)
     tmpfile_len = strlen(state->filename) + 6;
     tmpfile = calloc(1, tmpfile_len);
     snprintf (tmpfile, tmpfile_len, "%s.tmp", state->filename);
-    if (stat (tmpfile, &file_info) == 0)
-    {
+    if (stat (tmpfile, &file_info) == 0) {
         ICECAST_LOG_WARN("temp file \"%s\" exists, rejecting operation", tmpfile);
         free (tmpfile);
         fclose (passwdfile);
@@ -358,7 +330,7 @@ static auth_result htpasswd_deleteuser(auth_t *auth, const char *username)
     tmp_passwdfile = fopen(tmpfile, "wb");
 
     if(tmp_passwdfile == NULL) {
-        ICECAST_LOG_WARN("Failed to open temporary authentication database \"%s\": %s", 
+        ICECAST_LOG_WARN("Failed to open temporary authentication database \"%s\": %s",
                 tmpfile, strerror(errno));
         fclose(passwdfile);
         free(tmpfile);
@@ -393,18 +365,17 @@ static auth_result htpasswd_deleteuser(auth_t *auth, const char *username)
     /* Windows won't let us rename a file if the destination file
        exists...so, lets remove the original first */
     if (remove(state->filename) != 0) {
-        ICECAST_LOG_ERROR("Problem moving temp authentication file to original \"%s\" - \"%s\": %s", 
+        ICECAST_LOG_ERROR("Problem moving temp authentication file to original \"%s\" - \"%s\": %s",
                 tmpfile, state->filename, strerror(errno));
-    }
-    else {
+    } else {
         if (rename(tmpfile, state->filename) != 0) {
-            ICECAST_LOG_ERROR("Problem moving temp authentication file to original \"%s\" - \"%s\": %s", 
+            ICECAST_LOG_ERROR("Problem moving temp authentication file to original \"%s\" - \"%s\": %s",
                     tmpfile, state->filename, strerror(errno));
         }
     }
     free(tmpfile);
-    thread_rwlock_unlock (&state->file_rwlock);
-    htpasswd_recheckfile (state);
+    thread_rwlock_unlock(&state->file_rwlock);
+    htpasswd_recheckfile(state);
 
     return AUTH_USERDELETED;
 }
@@ -418,29 +389,27 @@ static auth_result htpasswd_userlist(auth_t *auth, xmlNodePtr srcnode)
 
     state = auth->state;
 
-    if (state->filename == NULL) {
+    if (!state->filename) {
         ICECAST_LOG_ERROR("No filename given in options for authenticator.");
         return AUTH_FAILED;
     }
 
-    htpasswd_recheckfile (state);
+    htpasswd_recheckfile(state);
 
     if (state->users == NULL) {
         ICECAST_LOG_ERROR("No user list.");
         return AUTH_FAILED;
     }
 
-    thread_rwlock_rlock (&state->file_rwlock);
-    node = avl_get_first (state->users);
-    while (node)
-    {
+    thread_rwlock_rlock(&state->file_rwlock);
+    node = avl_get_first(state->users);
+    while (node) {
         htpasswd_user *user = (htpasswd_user *)node->key;
-        newnode = xmlNewChild (srcnode, NULL, XMLSTR("User"), NULL);
+        newnode = xmlNewChild(srcnode, NULL, XMLSTR("user"), NULL);
         xmlNewTextChild(newnode, NULL, XMLSTR("username"), XMLSTR(user->name));
-        node = avl_get_next (node);
+        node = avl_get_next(node);
     }
-    thread_rwlock_unlock (&state->file_rwlock);
+    thread_rwlock_unlock(&state->file_rwlock);
 
     return AUTH_OK;
 }
-
