@@ -38,6 +38,8 @@
 #define CONFIG_PROBLEM_OBSOLETE_NODE    0x0020U
 #define CONFIG_PROBLEM_INVALID_NODE     0x0040U
 #define CONFIG_PROBLEM_VALIDATION       0x0080U
+#define CONFIG_PROBLEM_FILE_PERMISSION  0x0100U
+#define CONFIG_PROBLEM_UNSAFE_FILE      0x0200U
 
 typedef enum _http_header_type {
     /* static: headers are passed as is to the client. */
@@ -79,6 +81,13 @@ typedef enum {
     FALLBACK_OVERRIDE_OWN
 } fallback_override_t;
 
+typedef enum {
+    INTERPOLATION_DEFAULT = 0,
+    INTERPOLATION_NONE,
+    INTERPOLATION_STRFTIME,
+    INTERPOLATION_UUID
+} interpolation_t;
+
 typedef struct _mount_proxy {
     /* The mountpoint this proxy is used for */
     char *mountname;
@@ -88,6 +97,9 @@ typedef struct _mount_proxy {
      * NULL to not dump.
      */
     char *dumpfile;
+    interpolation_t dumpfile_interpolation;
+    uint64_t dumpfile_size_limit;
+    unsigned int dumpfile_time_limit;
     /* Send contents of file to client before the stream */
     char *intro_filename;
     /* Switch new listener to fallback source when max listeners reached */
@@ -184,6 +196,9 @@ typedef struct _listener_t {
     char *bind_address;
     int shoutcast_compat;
     char *shoutcast_mount;
+    char *client_address; /* for display only */
+    char *client_ip;      /* for actual work */
+    char *trusted_proxy[8];
     tlsmode_t tls;
     auth_stack_t *authstack;
     /* additional HTTP headers */
@@ -247,6 +262,7 @@ struct ice_config_tag {
     int header_timeout;
     int source_timeout;
     int body_timeout;
+    int server_idle_timeout;
     int fileserve;
     int on_demand; /* global setting for all relays */
 
@@ -285,6 +301,7 @@ struct ice_config_tag {
     char *allowfile;
     char *webroot_dir;
     char *adminroot_dir;
+    char *geoipdbfile;
     prng_seed_config_t *prng_seed;
     resource_t *resources;
     reportxml_database_t *reportxml_db;
