@@ -26,6 +26,9 @@
 
 #include <libxml/tree.h>
 
+#include "icecasttypes.h"
+#include <igloo/ro.h>
+
 #include "common/thread/thread.h"
 #include "common/avl/avl.h"
 #include "common/httpp/httpp.h"
@@ -54,6 +57,7 @@
 #include "acl.h"
 #include "listensocket.h"
 #include "fastevent.h"
+#include "geoip.h"
 
 /* for ADMIN_COMMAND_ERROR, and ADMIN_ICESTATS_LEGACY_EXTENSION_APPLICATION */
 #include "admin.h"
@@ -186,6 +190,8 @@ int client_create(client_t **c_ptr, connection_t *con, http_parser_t *parser)
 
     fastevent_emit(FASTEVENT_TYPE_CLIENT_CREATE, FASTEVENT_FLAG_MODIFICATION_ALLOWED, FASTEVENT_DATATYPE_CLIENT, client);
 
+    geoip_lookup_client(global.geoip_db, client);
+
     return ret;
 }
 
@@ -297,7 +303,7 @@ static inline void client_reuseconnection(client_t *client) {
 void client_destroy(client_t *client)
 {
     if (client == NULL) {
-        ICECAST_LOG_ERROR("Called with client=NULL. This is a BUG.");
+        ICECAST_LOG_DERROR("Called with client=NULL. This is a BUG.");
         return;
     }
 
@@ -310,10 +316,15 @@ void client_destroy(client_t *client)
     avl_tree_unlock(global_client_list);
 
     if (client->reuse != ICECAST_REUSE_CLOSE && !client->con->error) {
-        /* only reuse the client if we reached the body's EOF. */
-        if (client_body_eof(client) == 1) {
-            client_reuseconnection(client);
-            return;
+        const char * connection = httpp_getvar(client->parser, "connection");
+
+        /* check if the client asked us to close the connection. Better close it. Some clients (at least PHP) breaks otherwise */
+        if (connection && util_strcasestr(connection, "close") == NULL) {
+            /* only reuse the client if we reached the body's EOF. */
+            if (client_body_eof(client) == 1) {
+                client_reuseconnection(client);
+                return;
+            }
         }
     }
 
@@ -345,7 +356,7 @@ void client_destroy(client_t *client)
     if (client->free_client_data)
         client->free_client_data(client);
 
-    refobject_unref(client->handler_module);
+    igloo_ro_unref(&(client->handler_module));
     free(client->handler_function);
     free(client->uri);
     free(client->username);
@@ -385,12 +396,13 @@ int client_read_bytes(client_t *client, void *buf, unsigned len)
 
     if (client->encoding) {
         bytes = httpp_encoding_read(client->encoding, buf, len, reader, userdata);
+        ICECAST_LOG_DDEBUG("Reading with encoding from client %p with connection %p (connection ID: %llu, sock=%R) with buf=%p and len=%u returning %i", client, client->con, (long long unsigned int)client->con->id, client->con->sock, buf, len, bytes);
     } else {
         bytes = reader(userdata, buf, len);
     }
 
     if (bytes == -1 && client->con->error)
-        ICECAST_LOG_DEBUG("reading from connection has failed");
+        ICECAST_LOG_DEBUG("reading from connection has failed (client=%p, con=%p)", client, client->con);
 
     fastevent_emit(FASTEVENT_TYPE_CLIENT_READ, FASTEVENT_FLAG_MODIFICATION_ALLOWED, FASTEVENT_DATATYPE_OBRD, client, buf, (size_t)len, (ssize_t)bytes);
 
@@ -1015,6 +1027,7 @@ client_slurp_result_t client_body_skip(client_t *client)
 {
     char buf[2048];
     int ret;
+    ssize_t got;
 
     ICECAST_LOG_DEBUG("Slurping client %p", client);
 
@@ -1034,8 +1047,17 @@ client_slurp_result_t client_body_skip(client_t *client)
         if (left > sizeof(buf))
             left = sizeof(buf);
 
-        client_body_read(client, buf, left);
+        got = client_body_read(client, buf, left);
+    } else {
+        got = client_body_read(client, buf, sizeof(buf));
+    }
 
+    if (got < 1) {
+        ICECAST_LOG_DEBUG("Slurping client %p ... got no data EOF or needs more data", client);
+        return CLIENT_SLURP_ERROR;
+    }
+
+    if (client->request_body_length != -1) {
         if ((size_t)client->request_body_length == client->request_body_read) {
             ICECAST_LOG_DEBUG("Slurping client %p ... was a success", client);
             return CLIENT_SLURP_SUCCESS;
@@ -1043,8 +1065,6 @@ client_slurp_result_t client_body_skip(client_t *client)
             ICECAST_LOG_DEBUG("Slurping client %p ... needs more data", client);
             return CLIENT_SLURP_NEEDS_MORE_DATA;
         }
-    } else {
-        client_body_read(client, buf, sizeof(buf));
     }
 
     ret = client_body_eof(client);
@@ -1092,7 +1112,9 @@ ssize_t client_get_baseurl(client_t *client, listensocket_t *listensocket, char
 
         /* at least a couple of players (fb2k/winamp) are reported to send a
          * host header but without the port number. So if we are missing the
-         * port then lets treat it as if no host line was sent */
+         * port then lets treat it as if no host line was sent.
+         * It is also expected that any client using an IPv6 address here is
+         * either broken beyond repair or not affected. */
         if (host && strchr(host, ':') == NULL)
             host = NULL;
 
