@@ -3,11 +3,12 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org,
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2014-2018, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 
@@ -21,8 +22,6 @@
 #include <ogg/ogg.h>
 #include <string.h>
 
-typedef struct source_tag source_t;
-
 #include "refbuf.h"
 #include "format_ogg.h"
 #include "client.h"
@@ -41,7 +40,7 @@ static void midi_codec_free (ogg_state_t *ogg_info, ogg_codec_t *codec)
 
 
 /* Here, we just verify the page is ok and then add it to the queue */
-static refbuf_t *process_midi_page (ogg_state_t *ogg_info, ogg_codec_t *codec, ogg_page *page)
+static refbuf_t *process_midi_page (ogg_state_t *ogg_info, ogg_codec_t *codec, ogg_page *page, format_plugin_t *plugin)
 {
     refbuf_t * refbuf;
 
@@ -84,12 +83,12 @@ ogg_codec_t *initial_midi_page (format_plugin_t *plugin, ogg_page *page)
         codec->headers = 1;
         codec->name = "MIDI";
 
-        format_ogg_attach_header (ogg_info, page);
+        format_ogg_attach_header(ogg_info, page);
         return codec;
     } while (0);
 
-    ogg_stream_clear (&codec->os);
-    free (codec);
+    ogg_stream_clear(&codec->os);
+    free(codec);
     return NULL;
 }
 
