@@ -8,24 +8,26 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2014,      Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* format_mp3.h
-**
-** mp3 format plugin
-**
-*/
+ **
+ ** mp3 format plugin
+ **
+ */
 #ifndef __FORMAT_MP3_H__
 #define __FORMAT_MP3_H__
 
+#define MP3_METADATA_TITLE  "X_ICY_TITLE"
+#define MP3_METADATA_ARTIST "X_ICY_ARTIST"
+#define MP3_METADATA_URL    "X_ICY_URL"
+
 typedef struct {
     /* These are for inline metadata */
     int inline_metadata_interval;
     int offset;
     int interval;
-    char *url_artist;
-    char *url_title;
-    char *url;
     char *inline_url;
     int update_metadata;
 
