@@ -1,13 +1,14 @@
 /* Icecast
- *  
+ *
  *  This program is distributed under the GNU General Public License, version 2.
  *  A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org>, 
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org>,
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2014-2018, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 
@@ -21,8 +22,6 @@
 #include <ogg/ogg.h>
 #include <speex/speex_header.h>
 
-typedef struct source_tag source_t;
-
 #include "format_speex.h"
 #include "refbuf.h"
 #include "client.h"
@@ -38,7 +37,7 @@ static void speex_codec_free (ogg_state_t *ogg_info, ogg_codec_t *codec)
 
 
 static refbuf_t *process_speex_page (ogg_state_t *ogg_info,
-        ogg_codec_t *codec, ogg_page *page)
+        ogg_codec_t *codec, ogg_page *page, format_plugin_t *plugin)
 {
     refbuf_t *refbuf;
 
@@ -83,6 +82,9 @@ ogg_codec_t *initial_speex_page (format_plugin_t *plugin, ogg_page *page)
      * again for something else.
      */
     if (packet.bytes < 80) {
+        ICECAST_LOG_DDEBUG("Header too small for Speex, so skipping Speex test.");
+        ogg_stream_clear (&codec->os);
+        free (codec);
         return NULL;
     }
 
