@@ -3,12 +3,12 @@
  * This program is distributed under the GNU General Public License, version 2.
  * A copy of this license is included with this source.
  *
- * Copyright 2000-2004, Jack Moffitt <jack@xiph.org, 
+ * Copyright 2000-2004, Jack Moffitt <jack@xiph.org,
  *                      Michael Smith <msmith@xiph.org>,
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
+ * Copyright 2011-2020, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifdef HAVE_CONFIG_H
@@ -19,18 +19,20 @@
 #include <time.h>
 #include <string.h>
 
-#include "thread/thread.h"
-#include "httpp/httpp.h"
+#include "common/thread/thread.h"
+#include "common/httpp/httpp.h"
 
+#include "logging.h"
 #include "connection.h"
 #include "refbuf.h"
 #include "client.h"
 
 #include "compat.h"
 #include "cfgfile.h"
-#include "logging.h"
 #include "util.h"
 
+#define CATMODULE "logging"
+
 #ifdef _WIN32
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
@@ -41,13 +43,30 @@ int errorlog = 0;
 int accesslog = 0;
 int playlistlog = 0;
 
+int logging_str2logid(const char *str)
+{
+    if (!str)
+        return -1;
+
+    if (!strcmp(str, "error")) {
+        return errorlog;
+    } else if (!strcmp(str, "access")) {
+        return accesslog;
+    } else if (!strcmp(str, "playlist")) {
+        return playlistlog;
+    }
+
+    return -1;
+}
+
+
 #ifdef _WIN32
 /* Since strftime's %z option on win32 is different, we need
-   to go through a few loops to get the same info as %z */
+ to go through a few loops to get the same info as %z */
 int get_clf_time (char *buffer, unsigned len, struct tm *t)
 {
-    char    sign;
-    char    *timezone_string;
+    char sign;
+    char timezone_string[7];
     struct tm gmt;
     time_t time1 = time(NULL);
     int time_days, time_hours, time_tz;
@@ -65,6 +84,11 @@ int get_clf_time (char *buffer, unsigned len, struct tm *t)
 #endif
     /* FIXME: bail out if gmtime* returns NULL */
 
+    if (!thetime) {
+        snprintf(buffer, len, "<<BAD TIMESTAMP>>");
+        return 0;
+    }
+
     time_days = t->tm_yday - gmt.tm_yday;
 
     if (time_days < -1) {
@@ -91,32 +115,30 @@ int get_clf_time (char *buffer, unsigned len, struct tm *t)
         sign = '+';
     }
 
-    timezone_string = calloc(1, 7);
-    snprintf(timezone_string, 7, " %c%.2d%.2d", sign, time_tz / 60, time_tz % 60);
+    snprintf(timezone_string, sizeof(timezone_string), " %c%.2d%.2d", sign, time_tz / 60, time_tz % 60);
 
     now = time(NULL);
 
     thetime = localtime(&now);
-    strftime (buffer, len-7, "%d/%b/%Y:%H:%M:%S", thetime);
+    strftime(buffer, len - sizeof(timezone_string), "%d/%b/%Y:%H:%M:%S", thetime);
     strcat(buffer, timezone_string);
-	free(timezone_string);
     return 1;
 }
 #endif
-/* 
-** ADDR IDENT USER DATE REQUEST CODE BYTES REFERER AGENT [TIME]
-**
-** ADDR = client->con->ip
-** IDENT = always - , we don't support it because it's useless
-** USER = client->username
-** DATE = _make_date(client->con->con_time)
-** REQUEST = build from client->parser
-** CODE = client->respcode
-** BYTES = client->con->sent_bytes
-** REFERER = get from client->parser
-** AGENT = get from client->parser
-** TIME = timing_get_time() - client->con->con_time
-*/
+/*
+ ** ADDR IDENT USER DATE REQUEST CODE BYTES REFERER AGENT [TIME]
+ **
+ ** ADDR = client->con->ip
+ ** IDENT = always - , we don't support it because it's useless
+ ** USER = client->username
+ ** DATE = _make_date(client->con->con_time)
+ ** REQUEST = build from client->parser
+ ** CODE = client->respcode
+ ** BYTES = client->con->sent_bytes
+ ** REFERER = get from client->parser
+ ** AGENT = get from client->parser
+ ** TIME = timing_get_time() - client->con->con_time
+ */
 void logging_access(client_t *client)
 {
     char datebuf[128];
@@ -139,7 +161,7 @@ void logging_access(client_t *client)
     stayed = now - client->con->con_time;
 
     if (client->username == NULL)
-        username = "-"; 
+        username = "-";
     else
         username = client->username;
 
@@ -198,6 +220,11 @@ void logging_playlist(const char *mount, const char *metadata, long listeners)
              metadata);
 }
 
+void logging_mark(const char *username, const char *role)
+{
+    ICECAST_LOG_INFO("######## -- MARK -- (requested by %#H with role %#H) ########", username, role);
+    logging_playlist("/admin/", "-- MARK --", 0);
+}
 
 void log_parse_failure (void *ctx, const char *fmt, ...)
 {
@@ -246,4 +273,8 @@ void restart_logging (ice_config_t *config)
         log_set_archive_timestamp (playlistlog, config->logarchive);
         log_reopen (playlistlog);
     }
+
+    log_set_lines_kept(errorlog, config->error_log_lines_kept);
+    log_set_lines_kept(accesslog, config->access_log_lines_kept);
+    log_set_lines_kept(playlistlog, config->playlist_log_lines_kept);
 }
