@@ -8,36 +8,21 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2018,      Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifndef __SLAVE_H__
 #define __SLAVE_H__
 
-#include "thread/thread.h"
-
-typedef struct _relay_server {
-    char *server;
-    int port;
-    char *mount;
-    char *username;
-    char *password;
-    char *localmount;
-    char *bind;
-    struct source_tag *source;
-    int mp3metadata;
-    int on_demand;
-    int running;
-    int cleanup;
-    time_t start;
-    thread_type *thread;
-    struct _relay_server *next;
-} relay_server;
-
+#include "common/thread/thread.h"
+#include "icecasttypes.h"
+#include "cfgfile.h"
 
 void slave_initialize(void);
 void slave_shutdown(void);
 void slave_update_all_mounts (void);
 void slave_rebuild_mounts (void);
-relay_server *relay_free (relay_server *relay);
+void relay_config_free (relay_config_t *relay);
+relay_t *relay_free (relay_t *relay);
 
 #endif  /* __SLAVE_H__ */
