@@ -39,6 +39,10 @@
 #define snprintf _snprintf
 #endif
 
+#include <igloo/uuid.h>
+#include <igloo/sp.h>
+#include <igloo/error.h>
+
 #include "common/thread/thread.h"
 #include "common/avl/avl.h"
 #include "common/httpp/httpp.h"
@@ -100,6 +104,12 @@ source_t *source_reserve (const char *mount)
         if (src == NULL)
             break;
 
+        if (igloo_uuid_new_random_sp(&src->instance_uuid, igloo_instance) != igloo_ERROR_NONE) {
+            free(src);
+            src = NULL;
+            break;
+        }
+
         src->client_tree = avl_tree_new(client_compare, NULL);
         src->pending_tree = avl_tree_new(client_compare, NULL);
         src->history = playlist_new(10 /* DOCUMENT: default is max_tracks=10. */);
@@ -116,6 +126,11 @@ source_t *source_reserve (const char *mount)
     } while (0);
 
     avl_tree_unlock(global.source_tree);
+
+    if (src) {
+        stats_event(mount, "instance_uuid", src->instance_uuid);
+    }
+
     return src;
 }
 
@@ -214,7 +229,7 @@ void source_clear_source (source_t *source)
 {
     int c;
 
-    ICECAST_LOG_DEBUG("clearing source \"%s\"", source->mount);
+    ICECAST_LOG_DEBUG("clearing source %#H", source->mount);
 
     avl_tree_wlock (source->pending_tree);
     client_destroy(source->client);
@@ -222,15 +237,19 @@ void source_clear_source (source_t *source)
     source->parser = NULL;
     source->con = NULL;
 
+    if (source->format) {
+        if (igloo_sp_unref(&(source->format->contenttype), igloo_instance) != igloo_ERROR_NONE) {
+            ICECAST_LOG_ERROR("Cannot unref content type for source %#H", source->mount);
+        }
+    }
+
     /* log bytes read in access log */
     if (source->client && source->format)
         source->client->con->sent_bytes = source->format->read_bytes;
 
-    if (source->dumpfile)
-    {
+    if (source->dumpfile) {
         ICECAST_LOG_INFO("Closing dumpfile for %s", source->mount);
-        fclose (source->dumpfile);
-        source->dumpfile = NULL;
+        source_kill_dumpfile(source);
     }
 
     /* lets kick off any clients that are left on here */
@@ -289,7 +308,9 @@ void source_clear_source (source_t *source)
     source->prev_listeners = 0;
     source->hidden = 0;
     source->shoutcast_compat = 0;
-    source->client_stats_update = 0;
+    source->last_stats_update = 0;
+    source->create_time = 0;
+    source->flags = 0;
     util_dict_free(source->audio_info);
     source->audio_info = NULL;
 
@@ -328,6 +349,7 @@ void source_free_source (source_t *source)
     yp_remove (source->mount);
 
     refobject_unref(source->identifier);
+    igloo_sp_unref(&source->instance_uuid, igloo_instance);
     free (source->mount);
     free (source);
 
@@ -458,7 +480,7 @@ void source_move_clients(source_t *source, source_t *dest, connection_id_t *id,
             }
         }
 
-        ICECAST_LOG_INFO("passing %lu listeners to \"%s\"", count, dest->mount);
+        ICECAST_LOG_INFO("passing %lu listeners from % #H to % #H", count, source->mount, dest->mount);
 
         source->listeners -= count;
         stats_event_sub(source->mount, "listeners", count);
@@ -487,53 +509,55 @@ static refbuf_t *get_next_buffer (source_t *source)
 
     if (source->short_delay)
         delay = 0;
-    while (global.running == ICECAST_RUNNING && source->running)
-    {
-        int fds = 0;
-        time_t current = time (NULL);
 
-        if (source->client)
-            fds = util_timed_wait_for_fd (source->con->sock, delay);
-        else
-        {
-            thread_sleep (delay*1000);
+    while (global.running == ICECAST_RUNNING && source->running) {
+        int fds = 0;
+        time_t current = time(NULL);
+
+        if (source->client) {
+            fds = util_timed_wait_for_fd(source->con->sock, delay);
+        } else {
+            thread_sleep(delay*1000);
             source->last_read = current;
         }
 
-        if (current >= source->client_stats_update)
-        {
-            stats_event_args (source->mount, "total_bytes_read",
-                    "%"PRIu64, source->format->read_bytes);
-            stats_event_args (source->mount, "total_bytes_sent",
-                    "%"PRIu64, source->format->sent_bytes);
-            source->client_stats_update = current + 5;
+        if (current >= (source->last_stats_update + 5)) {
+            time_t age = current - source->create_time;
+
+            stats_event_args(source->mount, "total_bytes_read", "%"PRIu64, source->format->read_bytes);
+            stats_event_args(source->mount, "total_bytes_sent", "%"PRIu64, source->format->sent_bytes);
+            if (source->dumpfile) {
+                stats_event_args(source->mount, "dumpfile_written", "%"PRIu64, source->dumpfile_written);
+            }
+
+            if (age > 30) { /* TODO: Should this be configurable? */
+                source_set_flags(source, SOURCE_FLAG_AGED);
+            }
+
+            source->last_stats_update = current;
         }
-        if (fds < 0)
-        {
-            if (! sock_recoverable (sock_error()))
-            {
-                ICECAST_LOG_WARN("Error while waiting on socket, Disconnecting source");
+        if (fds < 0) {
+            if (!sock_recoverable(sock_error())) {
+                ICECAST_LOG_WARN("Error while waiting on socket, Disconnecting source on %#H", source->mount);
                 source->running = 0;
             }
             break;
         }
-        if (fds == 0)
-        {
+        if (fds == 0) {
             thread_mutex_lock(&source->lock);
-            if ((source->last_read + (time_t)source->timeout) < current)
-            {
+            if ((source->last_read + (time_t)source->timeout) < current) {
                 ICECAST_LOG_DEBUG("last %ld, timeout %d, now %ld", (long)source->last_read,
                         source->timeout, (long)current);
-                ICECAST_LOG_WARN("Disconnecting source due to socket timeout");
+                ICECAST_LOG_WARN("Disconnecting source on %#H due to socket timeout", source->mount);
                 source->running = 0;
             }
             thread_mutex_unlock(&source->lock);
             break;
         }
         source->last_read = current;
-        refbuf = source->format->get_buffer (source);
+        refbuf = source->format->get_buffer(source);
         if (client_body_eof(source->client)) {
-            ICECAST_LOG_INFO("End of Stream %s", source->mount);
+            ICECAST_LOG_INFO("End of Stream %H", source->mount);
             source->running = 0;
             continue;
         }
@@ -541,6 +565,9 @@ static refbuf_t *get_next_buffer (source_t *source)
             break;
     }
 
+    if (refbuf)
+        source_set_flags(source, SOURCE_FLAG_GOT_DATA);
+
     return refbuf;
 }
 
@@ -607,24 +634,88 @@ static void send_to_listener (source_t *source, client_t *client, int deletion_e
 /* Open the file for stream dumping.
  * This function should do all processing of the filename.
  */
-static FILE * source_open_dumpfile(const char * filename) {
-#ifndef _WIN32
-    /* some of the below functions seems not to be standard winapi functions */
+static bool source_open_dumpfile(source_t *source)
+{
+    const char *filename = source->dumpfilename;
+    interpolation_t interpolation = source->dumpfile_interpolation;
+    time_t curtime = time(NULL);
     char buffer[PATH_MAX];
-    time_t curtime;
-    struct tm *loctime;
 
-    /* Get the current time. */
-    curtime = time (NULL);
+    if (!filename) {
+        ICECAST_LOG_WARN("Can not open dump file for source %#H. No filename defined.", source->mount);
+        event_emit_va("dumpfile-error", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+        return false;
+    }
 
-    /* Convert it to local time representation. */
-    loctime = localtime (&curtime);
+    if (source->dumpfile) {
+        ICECAST_LOG_WARN("Can not open dump file for source %#H. Dump already running.", source->mount);
+        event_emit_va("dumpfile-error", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+        return false;
+    }
 
-    strftime (buffer, sizeof(buffer), filename, loctime);
-    filename = buffer;
+    if (!source->format->write_buf_to_file) {
+        ICECAST_LOG_WARN("Can not open dump file for source %#H. Format does not support dumping.", source->mount);
+        event_emit_va("dumpfile-error", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+        return false;
+    }
+
+    ICECAST_LOG_DDEBUG("source=%p{.mount=%#H, .burst_point=%p, .stream_data=%p, .stream_data_tail=%p, ...}", source, source->mount, source->burst_point, source->stream_data, source->stream_data_tail);
+
+    if (interpolation == INTERPOLATION_DEFAULT) {
+#ifndef _WIN32
+        interpolation = INTERPOLATION_STRFTIME;
+#else
+        interpolation = INTERPOLATION_NONE;
 #endif
+    }
 
-    return fopen (filename, "ab");
+    switch (interpolation) {
+        case INTERPOLATION_NONE:
+            /* no-op */
+            break;
+        case INTERPOLATION_STRFTIME:
+#ifndef _WIN32
+            {
+                /* some of the below functions seems not to be standard winapi functions */
+                struct tm *loctime;
+
+                /* Convert it to local time representation. */
+                loctime = localtime(&curtime);
+
+                strftime(buffer, sizeof(buffer), filename, loctime);
+                filename = buffer;
+            }
+#else
+            ICECAST_LOG_ERROR("Can not interpolation dump file filename for source %#H. strftime() interpolation is not supported by your operating system.", source->mount);
+#endif
+            break;
+        case INTERPOLATION_UUID:
+            if (!util_interpolation_uuid(buffer, sizeof(buffer), filename)) {
+                ICECAST_LOG_WARN("Can not open dump file for source %#H. Filename does not interpolate.", source->mount);
+                event_emit_va("dumpfile-error", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+                return false;
+            }
+            filename = buffer;
+            break;
+        default:
+            ICECAST_LOG_ERROR("Bug. Bad interpolation %i. Source %p on %#H.", (int)interpolation, source, source->mount);
+            break;
+    }
+
+    source->dumpfile = fopen(filename, "ab");
+
+    if (source->dumpfile) {
+        source->dumpfile_start = curtime;
+        stats_event(source->mount, "dumpfile_written", "0");
+        stats_event_time_iso8601(source->mount, "dumpfile_start");
+        event_emit_va("dumpfile-opened", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_KEY_DUMPFILE_FILENAME, filename, EVENT_EXTRA_LIST_END);
+        return true;
+    } else {
+        ICECAST_LOG_WARN("Cannot open dump file for source %#H with filename %#H for appending: %s, disabling.",
+                source->mount, source->dumpfilename, strerror(errno));
+        event_emit_va("dumpfile-error", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_KEY_DUMPFILE_FILENAME, filename, EVENT_EXTRA_LIST_END);
+        return false;
+    }
 }
 
 /* Perform any initialisation just before the stream data is processed, the header
@@ -634,6 +725,7 @@ static void source_init (source_t *source)
 {
     char listenurl[512];
     const char *str;
+    time_t now;
 
     str = httpp_getvar(source->parser, "ice-audio-info");
     source->audio_info = util_dict_new();
@@ -647,14 +739,7 @@ static void source_init (source_t *source)
     stats_event (source->mount, "listenurl", listenurl);
 
     if (source->dumpfilename != NULL)
-    {
-        source->dumpfile = source_open_dumpfile (source->dumpfilename);
-        if (source->dumpfile == NULL)
-        {
-            ICECAST_LOG_WARN("Cannot open dump file \"%s\" for appending: %s, disabling.",
-                    source->dumpfilename, strerror(errno));
-        }
-    }
+        source_open_dumpfile(source);
 
     /* grab a read lock, to make sure we get a chance to cleanup */
     thread_rwlock_rlock (source->shutdown_rwlock);
@@ -668,12 +753,15 @@ static void source_init (source_t *source)
     stats_event_time (source->mount, "stream_start");
     stats_event_time_iso8601 (source->mount, "stream_start_iso8601");
 
+    now = time(NULL);
+    source->last_read = now;
+    source->create_time = now;
+
     ICECAST_LOG_DEBUG("Source creation complete");
-    source->last_read = time (NULL);
     source->prev_listeners = -1;
     source->running = 1;
 
-    event_emit_clientevent("source-connect", source->client, source->mount);
+    event_emit_va("source-connect", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_CLIENT, source->client, EVENT_EXTRA_LIST_END);
 
     /*
     ** Now, if we have a fallback source and override is on, we want
@@ -712,12 +800,12 @@ static void source_init (source_t *source)
 void source_main (source_t *source)
 {
     refbuf_t *refbuf;
-    client_t *client;
     avl_node *client_node;
 
     source_init (source);
 
     while (global.running == ICECAST_RUNNING && source->running) {
+        source_flags_t old_flags;
         int remove_from_q;
 
         refbuf = get_next_buffer (source);
@@ -774,7 +862,7 @@ void source_main (source_t *source)
 
         client_node = avl_get_first(source->client_tree);
         while (client_node) {
-            client = (client_t *) client_node->key;
+            client_t *client = (client_t *) client_node->key;
 
             send_to_listener(source, client, remove_from_q);
 
@@ -792,7 +880,9 @@ void source_main (source_t *source)
 
         /** add pending clients **/
         client_node = avl_get_first(source->pending_tree);
+        old_flags = source->flags;
         while (client_node) {
+            client_t *client = (client_t *)client_node->key;
 
             if(source->max_listeners != -1 &&
                     source->listeners >= (unsigned long)source->max_listeners)
@@ -802,7 +892,6 @@ void source_main (source_t *source)
                  * and doesn't give the listening client any information about
                  * why they were disconnected
                  */
-                client = (client_t *)client_node->key;
                 client_node = avl_get_next(client_node);
                 avl_delete(source->pending_tree, (void *)client, _free_client);
 
@@ -817,9 +906,28 @@ void source_main (source_t *source)
             source->listeners++;
             ICECAST_LOG_DEBUG("Client added for mountpoint (%s)", source->mount);
             stats_event_inc(source->mount, "connections");
+            event_emit_va("source-listener-attach", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_CLIENT, client, EVENT_EXTRA_LIST_END);
+
+            {
+                http_parser_t *parser = client->parser;
+                const char *protocol = httpp_getvar(parser, HTTPP_VAR_PROTOCOL);
+                const char *version = httpp_getvar(parser, HTTPP_VAR_VERSION);
+                const char *http_host = httpp_getvar(parser, "host");
+
+                if (protocol && version) {
+                    if (strcasecmp(protocol, "HTTP") == 0 && strcmp(version, "1.0") == 0) {
+                        source->flags |= SOURCE_FLAG_HTTP_1_0_LISTENER;
+                    }
+                }
+
+                if (!http_host || !strchr(http_host, ':'))
+                    source->flags |= SOURCE_FLAG_NOHOST_LISTENER;
+            }
 
             client_node = avl_get_next(client_node);
         }
+        if (old_flags != source->flags)
+            event_emit_va("source-flags-changed", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
 
         /** clear pending tree **/
         while (avl_get_first(source->pending_tree)) {
@@ -844,6 +952,11 @@ void source_main (source_t *source)
             stats_event_args (source->mount, "listeners", "%lu", source->listeners);
             if (source->listeners == 0 && source->on_demand)
                 source->running = 0;
+
+            if (source->listeners == 0)
+                event_emit_va("source-listeners-is-zero", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+
+            event_emit_va("source-listeners-changed", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
         }
 
         /* lets reduce the queue, any lagging clients should of been
@@ -883,12 +996,12 @@ static void source_shutdown (source_t *source)
 {
     source->running = 0;
     if (source->con && source->con->ip) {
-        ICECAST_LOG_INFO("Source from %s at \"%s\" exiting", source->con->ip, source->mount);
+        ICECAST_LOG_INFO("Source from %s at %#H exiting", source->con->ip, source->mount);
     } else {
-        ICECAST_LOG_INFO("Source at \"%s\" exiting", source->mount);
+        ICECAST_LOG_INFO("Source at %#H exiting", source->mount);
     }
 
-    event_emit_clientevent("source-disconnect", source->client, source->mount);
+    event_emit_va("source-disconnect", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_CLIENT, source->client, EVENT_EXTRA_LIST_END);
 
     /* we have de-activated the source now, so no more clients will be
      * added, now move the listeners we have to the fallback (if any)
@@ -914,13 +1027,7 @@ static void source_shutdown (source_t *source)
         const char *expectcontinue = httpp_getvar(source->parser, "expect");
 
         if (expectcontinue != NULL) {
-#ifdef HAVE_STRCASESTR
-            if (strcasestr (expectcontinue, "100-continue") != NULL)
-#else
-            ICECAST_LOG_WARN("OS doesn't support case insensitive substring checks...");
-            if (strstr (expectcontinue, "100-continue") != NULL)
-#endif
-            {
+            if (util_strcasestr(expectcontinue, "100-continue") != NULL) {
                 client_t *client = source->client;
                 source->client = NULL; /* detach client from source. */
 
@@ -940,6 +1047,7 @@ static void source_shutdown (source_t *source)
 
     global_lock();
     global.sources--;
+    global.sources_update = time(NULL);
     stats_event_args(NULL, "sources", "%d", global.sources);
     global_unlock();
 
@@ -1011,11 +1119,14 @@ static void source_apply_mount (ice_config_t *config, source_t *source, mount_pr
     int val;
     http_parser_t *parser = NULL;
     acl_t *acl = NULL;
+    source_flags_t old_flags = source->flags;
 
     ICECAST_LOG_DEBUG("Applying mount information for \"%s\"", source->mount);
     avl_tree_rlock (source->client_tree);
     stats_event_args (source->mount, "listener_peak", "%lu", source->peak_listeners);
 
+    source->flags &= ~SOURCE_FLAGS_CLEARABLE;
+
     if (mountinfo)
     {
         source->max_listeners = mountinfo->max_listeners;
@@ -1152,6 +1263,12 @@ static void source_apply_mount (ice_config_t *config, source_t *source, mount_pr
     }
     stats_event (source->mount, "bitrate", str);
 
+    if ((str = httpp_getvar(parser, "content-language"))) {
+        stats_event(source->mount, "content-language", str);
+    } else {
+        source->flags |= SOURCE_FLAG_NO_LANGUAGE;
+    }
+
     /* handle MIME-type */
     if (mountinfo && mountinfo->type)
         stats_event (source->mount, "server_type", mountinfo->type);
@@ -1172,23 +1289,35 @@ static void source_apply_mount (ice_config_t *config, source_t *source, mount_pr
         stats_event (source->mount, "authenticator", NULL);
     acl_release(acl);
 
-    if (mountinfo && mountinfo->fallback_mount)
-    {
-        char *mount = source->fallback_mount;
-        source->fallback_mount = strdup (mountinfo->fallback_mount);
-        free (mount);
-    }
-    else
+    if (mountinfo && mountinfo->fallback_mount) {
+        util_replace_string(&(source->fallback_mount), mountinfo->fallback_mount);
+    } else {
+        free(source->fallback_mount);
         source->fallback_mount = NULL;
-
-    if (mountinfo && mountinfo->dumpfile)
-    {
-        char *filename = source->dumpfilename;
-        source->dumpfilename = strdup (mountinfo->dumpfile);
-        free (filename);
     }
-    else
+
+    /* Dumpfile settings */
+    if (mountinfo && mountinfo->dumpfile) {
+        util_replace_string(&(source->dumpfilename), mountinfo->dumpfile);
+    } else {
+        free(source->dumpfilename);
         source->dumpfilename = NULL;
+        if (source->dumpfile) {
+            ICECAST_LOG_INFO("Stopping dumpfile as it is now de-configured for source %p at mountpoint %#H.", source, source->mount);
+            source_kill_dumpfile(source);
+        }
+    }
+
+    if (mountinfo) {
+        source->dumpfile_size_limit    = mountinfo->dumpfile_size_limit;
+        source->dumpfile_time_limit    = mountinfo->dumpfile_time_limit;
+        source->dumpfile_interpolation = mountinfo->dumpfile_interpolation;
+    } else {
+        source->dumpfile_size_limit    = 0;
+        source->dumpfile_time_limit    = 0;
+        source->dumpfile_interpolation = INTERPOLATION_DEFAULT;
+    }
+
 
     if (source->intro_file)
     {
@@ -1231,6 +1360,9 @@ static void source_apply_mount (ice_config_t *config, source_t *source, mount_pr
     if (mountinfo && mountinfo->max_history > 0)
         playlist_set_max_tracks(source->history, mountinfo->max_history);
 
+    if (old_flags != source->flags)
+        event_emit_va("source-flags-changed", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+
     avl_tree_unlock(source->client_tree);
 }
 
@@ -1465,3 +1597,94 @@ void source_recheck_mounts (int update_all)
     avl_tree_unlock (global.source_tree);
     config_release_config();
 }
+
+/* Writes a buffer of raw data to a dumpfile. returns true if the write was successful (and complete). */
+bool source_write_dumpfile(source_t *source, const void *buffer, size_t len)
+{
+    if (!source->dumpfile)
+        return false;
+
+    if (!len)
+        return true;
+
+    if (fwrite(buffer, 1, len, source->dumpfile) != len) {
+        ICECAST_LOG_WARN("Write to dump file failed, disabling");
+        source_kill_dumpfile(source);
+        return false;
+    }
+
+    source->dumpfile_written += len;
+
+    if (source->dumpfile_size_limit && source->dumpfile_written > source->dumpfile_size_limit) {
+        ICECAST_LOG_INFO("Dumpfile for source %p at mountpoint %#H reached size limit. Dumpfile will be disabled.", source, source->mount);
+        source_kill_dumpfile(source);
+    } else if (source->dumpfile_time_limit) {
+        time_t now = time(NULL);
+        if (now > (source->dumpfile_start + source->dumpfile_time_limit)) {
+            ICECAST_LOG_INFO("Dumpfile for source %p at mountpoint %#H reached time limit. Dumpfile will be disabled.", source, source->mount);
+            source_kill_dumpfile(source);
+        }
+    }
+
+    return true;
+}
+
+void source_kill_dumpfile(source_t *source)
+{
+    if (!source->dumpfile)
+        return;
+
+    if (source->format && source->format->on_file_close)
+        source->format->on_file_close(source);
+
+    fclose(source->dumpfile);
+    source->dumpfile = NULL;
+    source->dumpfile_written = 0;
+    stats_event(source->mount, "dumpfile_written", NULL);
+    stats_event(source->mount, "dumpfile_start", NULL);
+    event_emit_va("dumpfile-closed", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+}
+
+health_t source_get_health(source_t *source)
+{
+    const source_flags_t flags = source->flags;
+    return source_get_health_by_flags(flags);
+}
+
+health_t source_get_health_by_flags(source_flags_t flags)
+{
+    health_t health = HEALTH_OK;
+
+    if (!(flags & SOURCE_FLAG_GOT_DATA))
+        health = health_atbest(health, HEALTH_ERROR);
+
+    if (flags & SOURCE_FLAG_FORMAT_GENERIC)
+        health = health_atbest(health, HEALTH_WARNING);
+
+    if (flags & SOURCE_FLAG_LEGACY_METADATA)
+        health = health_atbest(health, HEALTH_ERROR);
+
+    if (!(flags & SOURCE_FLAG_AGED))
+        health = health_atbest(health, HEALTH_WARNING);
+
+    return health;
+}
+
+void source_set_flags(source_t *source, source_flags_t flags)
+{
+    bool changed;
+    source_flags_t old_flags;
+
+    /* check if we need to do anything at all */
+    if ((source->flags & flags) == flags)
+        return;
+
+    thread_mutex_lock(&source->lock);
+    old_flags = source->flags;
+    source->flags |= flags;
+    changed = old_flags != source->flags;
+    thread_mutex_unlock(&source->lock);
+
+    if (changed)
+        event_emit_va("source-flags-changed", EVENT_EXTRA_SOURCE, source, EVENT_EXTRA_LIST_END);
+}
