@@ -5,7 +5,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- *  Copyright 2012-2020 Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>
+ *  Copyright 2012-2022 Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>
  *
  *  This program is distributed under the GNU General Public License, version 2.
  *  A copy of this license is included with this source.
@@ -16,9 +16,11 @@
 
 /* for FILE* */
 #include <stdio.h>
+#include <stdbool.h>
 
 #include "common/net/sock.h"
 #include "icecasttypes.h"
+#include "util_string.h" /* so not all users need to be updated yet */
 
 #define UNKNOWN_CONTENT 0
 #define XSLT_CONTENT    1
@@ -32,13 +34,11 @@
 int util_timed_wait_for_fd(sock_t fd, int timeout);
 int util_read_header(sock_t sock, char *buff, unsigned long len, int entire);
 int util_check_valid_extension(const char *uri);
+int util_check_valid_ip(const char *ip);
 char *util_get_extension(const char *path);
 char *util_get_path_from_uri(char *uri);
 char *util_get_path_from_normalised_uri(const char *uri);
 char *util_normalise_uri(const char *uri);
-char *util_base64_encode(const char *data, size_t len);
-char *util_base64_decode(const char *input);
-char *util_bin_to_hex(unsigned char *data, int len);
 
 typedef enum _util_hostcheck_tag {
     HOSTCHECK_ERROR = -1,
@@ -57,9 +57,6 @@ int util_str_to_loglevel(const char *str);
 int util_str_to_int(const char *str, const int default_value);
 unsigned int util_str_to_unsigned_int(const char *str, const unsigned int default_value);
 
-char *util_url_unescape(const char *src);
-char *util_url_escape(const char *src);
-
 /* Function to build up a HTTP header.
  * out is the pointer to storage.
  * len is the total length of out.
@@ -129,6 +126,21 @@ char *util_conv_string (const char *string, const char *in_charset, const char *
 
 int get_line(FILE *file, char *buf, size_t siz);
 
-int util_replace_string(char **dst, const char *src);
-int util_strtolower(char *str);
+/* returns true on success, when returning false buffer[] is in undefined state. */
+bool util_interpolation_uuid(char * buffer, size_t bufferlen, const char *in);
+
+/* test for file modes */
+/* Returns true if any of the tests fail, also will log to error log */
+
+#define UTIL_TEST_FILE_MODE_NO_EXEC      ((uint32_t)0x00000001)
+#define UTIL_TEST_FILE_MODE_NO_PUB_WRITE ((uint32_t)0x00000002)
+#define UTIL_TEST_FILE_MODE_NO_PUB_READ  ((uint32_t)0x00000004)
+#define UTIL_TEST_FILE_MODE_DEFAULTS     ((uint32_t)0x80000000)
+
+#ifndef _WIN32
+bool util_test_file_modes(const char *filename, uint32_t tests);
+#else
+#define util_test_file_modes(filename,tests) false
+#endif
+
 #endif  /* __UTIL_H__ */
