@@ -8,7 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
- * Copyright 2014-2018, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
+ * Copyright 2014-2022, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* -*- c-basic-offset: 4; indent-tabs-mode: nil; -*- */
@@ -17,19 +17,6 @@
 
 #include "icecasttypes.h"
 
-#define  YP_SERVER_NAME         1
-#define  YP_SERVER_DESC         2
-#define  YP_SERVER_GENRE        3
-#define  YP_SERVER_URL          4
-#define  YP_BITRATE             5
-#define  YP_AUDIO_INFO          6
-#define  YP_SERVER_TYPE         7
-#define  YP_CURRENT_SONG        8
-#define  YP_CLUSTER_PASSWORD    9
-#define  YP_SUBTYPE             10 
-
-#define YP_ADD_ALL              -1
-
 #ifdef USE_YP
 void yp_add (const char *mount);
 void yp_remove (const char *mount);
